local _G = _G
local L = Loremaster.L
local GetCurrentMapContinent = GetCurrentMapContinent

local function createLDBMenu()

	local LDBMenu = CreateFrame("Frame", "LoremasterLDBMenu")
	
	LDBMenu.displayMode = "MENU"
	LDBMenu.info = {}
	LDBMenu.HideMenu = 
		function()
			if UIDROPDOWNMENU_OPEN_MENU == LDBMenu then
				CloseDropDownMenus()
			end
		end

	LDBMenu.initialize = 
		function(self, level)
		
			if not level then return end
			
			local info = self.info
			local t = L["Loremaster"]
						
			wipe(info)
			
			if level == 1 then
			
				info.isTitle = 1
				info.text = t
				info.notCheckable = 1
				UIDropDownMenu_AddButton(info, level)

				info.keepShownOnClick = 1
				info.disabled = nil
				info.isTitle = nil

				info.text = L["Show Loremaster"]
				info.func = function() self.HideMenu(); LoremasterMainFrame:Show() end
				UIDropDownMenu_AddButton(info, level)

				info.text = L["Hide Loremaster"]
				info.func = function() self.HideMenu(); LoremasterMainFrame:Hide() end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = _G.MAIN_MENU
				info.func = function() InterfaceOptionsFrame_OpenToCategory(_G.LoremasterOptionsMain) end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = ""
				info.func = nil
				UIDropDownMenu_AddButton(info,level)
				
				info.text = L["Reset Loremaster"]
				info.func = function() self.HideMenu(); StaticPopup_Show("LOREMASTER_RESET") end
				UIDropDownMenu_AddButton(info, level)

				info.text = ""
				info.func = nil
				UIDropDownMenu_AddButton(info,level)
				
				-- Close menu item
				info.text = _G.CLOSE
				info.func = self.HideMenu
				UIDropDownMenu_AddButton(info, level)
				
			end
			
		end

end

local function LDBOnClick(self, button)

	if button == "LeftButton" then

		if LoremasterMainFrame:IsVisible() then
		
			LoremasterMainFrame:Hide()
			
		else
		
			LoremasterMainFrame:Show()
			
		end

	else

		--drop down menu
		ToggleDropDownMenu(1, nil, LoremasterLDBMenu, self:GetName(), 0, 0)

	end

end

function Loremaster:questComplete()
	
	if Loremaster.qid then
	
		Loremaster:removeQuest(Loremaster.qid)
		Loremaster.qid = nil
		Loremaster.rescan = true
		
	end
			
end

function Loremaster:achscan()

	local zid
	
	Loremaster:scanAchievements()
	
	if LoremasterContinentFrame.currentline then
	
		zid = LoremasterZoneFrame.currentline
		Loremaster:clickResponder(LoremasterContinentFrame.currentline)
		
	end
	
	LoremasterZoneFrame.currentline = zid
	
	if LoremasterZoneFrame.currentline then
		
		Loremaster:clickResponder(LoremasterZoneFrame.currentline)
			
	end
		
end
-- *** watch events ***
function Loremaster:eventHandler(self, event, arg1, ...)

	local zid
	
	if event == "ADDON_LOADED" and arg1 == "Loremaster" then

		SLASH_LOREMASTER_CMD1 = "/" .. L["lm"]
		SLASH_LOREMASTER_CMD2 = "/" .. L["loremaster"]
		SlashCmdList["LOREMASTER_CMD"] = function(input) Loremaster:chatCommand(input) end

		if LoremasterDB == nil then

			LoremasterDB = {}
			
		end
		
		Loremaster.db = LibStub("AceDB-3.0"):New("LoremasterVars", Loremaster.defaults, "profile")
		Loremaster:createLoremasterOptions()

		--configure LDB
		createLDBMenu()
		ldblib = LibStub:GetLibrary("LibDataBroker-1.1")
		ldb = ldblib:NewDataObject(L["Loremaster"], 
		{
			type = "data source", 
			text = L["Loremaster"],
			icon = "Interface\\ICONS\\INV_Misc_Book_07",
			OnClick = LDBOnClick			
		})
		function ldb.OnTooltipShow(tip)
			tip:AddLine(L["Loremaster"])
			tip:AddLine(" ")
			tip:AddLine("|cffffffff" .. _G.GAME_VERSION_LABEL .. " " .. GetAddOnMetadata("Loremaster", "Version"))
		end

		Loremaster.TT = (TomTom ~= nil)
		Loremaster.LH = (LightHeaded ~= nil)
		
		if Loremaster.TT then
			
			--code hijacked from TomTom
			local cidx, c, zidx, z
			
			Loremaster.zlist = {}
			
			for cidx,c in ipairs{GetMapContinents()} do
			
				for zidx,z in ipairs{GetMapZones(cidx)} do
				
					Loremaster.zlist[z:lower():gsub("[%L]", "")] = {cidx, zidx, z}
					
				end
			
			end
			
		end
		
		--Add a button to Blizzard's Quest Log Frame
		local blizzquestlogframebutton = CreateFrame("Button", nil, QuestLogFrame, "UIPanelButtonTemplate")
		
		blizzquestlogframebutton:SetWidth(100)
		blizzquestlogframebutton:SetHeight(22)
		blizzquestlogframebutton:SetText(L["Loremaster"])
		blizzquestlogframebutton:SetPoint("LEFT", QuestLogControlPanel, "RIGHT", 0, 0)
		blizzquestlogframebutton:SetScript("OnClick", function() if LoremasterMainFrame:IsVisible() then LoremasterMainFrame:Hide(); else LoremasterMainFrame:Show(); end end)
	
	elseif event == "PLAYER_LOGIN" then
		
		Loremaster.faction = UnitFactionGroup("player")
		
		if Loremaster.faction == "Alliance" then
		
			Loremaster.factiontexture = "Interface\\GROUPFRAME\\UI-Group-PVP-Alliance"
			
		else
		
			Loremaster.factiontexture = "Interface\\GROUPFRAME\\UI-Group-PVP-Horde"
			
		end
		
		if (LoremasterDB.Achievements == nil) or Loremaster.rescan then
		
			Loremaster:scanAchievements()
			
		end
		
		Loremaster:setFrameLevel()
		Loremaster:CreateQuestFrame()
	
	elseif event == "QUEST_QUERY_COMPLETE" then

		if Loremaster.query then
		
			Loremaster:buildQuestList()
			Loremaster.query = nil
			
		end
	
	elseif event == "ACHIEVEMENT_EARNED" then

		Loremaster.rescan = true
		Loremaster:scanAchievements()
		
	elseif event == "QUEST_AUTOCOMPLETE" then

		Loremaster.qid = arg1
		Loremaster:questComplete()
				
	elseif event == "PLAYER_REGEN_DISABLED" then
	
		Loremaster.incombat = true
		
		if Loremaster.db.profile.hideincombat then
		
			if LoremasterMainFrame:IsVisible() then
			
				LoremasterMainFrame:Hide()
				
			end
			
		end
		
	elseif event == "PLAYER_REGEN_ENABLED" then
	
		Loremaster.incombat = nil
		
		if Loremaster.rescan then
		
			Loremaster:achscan()
			
		end
		
	elseif event == "CRITERIA_UPDATE" then
	
		if (not Loremaster.incombat) and LoremasterMainFrame then
		
			Loremaster:achscan()
			
		end
		
	end

end