local GetAchievementInfo = GetAchievementInfo
local GetAchievementCriteriaInfo = GetAchievementCriteriaInfo
local L = Loremaster.L
local continents = {"Kalimdor", "EasternKingdoms", "Outland", "Northrend", "Maelstrom"}
local locale = GetLocale()


function Loremaster:setFrameLevel()

	local childframes = {UIParent:GetChildren()}
	local highest = 0
	local fl, frame

	for _, frame in ipairs(childframes) do

		if frame:GetFrameStrata() == "MEDIUM" then
		
			fl = frame:GetFrameLevel()
			
			if fl > highest then highest = fl end
			
		end

	end

	Loremaster.framelevel = highest + 1
	
end

function Loremaster:resetLines(frame)

	local row
	
	for row = 1, frame.displaylines do
		
		_G[frame.name .. row]:SetBackdrop(nil)
			
	end
		
end

function Loremaster:getContinents()

	local cn, c, ach, det
	local dat = {}
	
	if Loremaster.db.profile.hideach then

		for cn, c in ipairs(Loremaster.continents) do
		
			for ach, det in pairs((LoremasterDB.Achievements[cn] or {})) do
			
				if det[3] == false then
				
					table.insert(dat, Loremaster.continents[cn])
					break
					
				end
				
			end
				
		end
			
	else
		
		dat = Loremaster.continents
			
	end
	
	return dat
	
end

function Loremaster:removeQuest(qid)

	local c, zone, qs
	local found, q, d
	
	if not LoremasterDB.available then return end
	
	for c = 1, 5 do
	
		for zone, qs in pairs((LoremasterDB.available[tostring(c)]) or {}) do
		
			--remove the quest if it exists
			for q, d in pairs(qs) do
			
				if q == qid then
				
					LoremasterDB.available[tostring(c)][zone][qid] = nil
					found = true
					break
					
				end
				
			end
			
			if found then
			
				break
				
			end
		
		end
		
	end
	
end

function Loremaster:updateQuestCount(count)

	local texture = "Interface\\OPTIONSFRAME\\UI-OptionsFrame-NewFeatureIcon"
	local size = 16
	local qcount
	
	if (count or -1) == -1 then
	
		qcount = ""
		
	else
	
		qcount = "  |cffbebebe(" .. count .. " " .. L["available"] .. ")"
		
	end
	
	LoremasterQuestTitle:SetFormattedText("|T%s:%d|t%s",texture , size, L["Quests"] .. qcount)

end

function Loremaster:updateZoneCount(count)

	local texture = Loremaster.factiontexture
	local size = 32
	local zcount
	
	if (count or -1) == -1 then
	
		zcount = ""
		
	else
	
		zcount = "  |cffbebebe(" .. count .. " " .. L["available"] .. ")"
		
	end
	
	LoremasterZoneTitle:SetFormattedText("|T%s:%d|t%s",texture , size, L["Achievement"] .. zcount)

end

function Loremaster:setNotes(detail)

	local N = LoremasterInfoFrame
	local white, gold, green, blue = "|cffffffff", "|cffffd200", "|cff22ee55", "|cff00bfff"
	local text, sp1, npc, item, zone, target
	
	text = "|T"
	
	if detail.i == 1 then
	
		text = text .. Loremaster.itemicon
		target = detail.drop
		item = L["an item dropped by"] .. " " .. green .. target .. white
		sp1 = " "

	else
	
		text = text .. Loremaster.npcicon
		item = L["an NPC"]
		sp1 = ""
		target = detail.npc
		
	end
	
	npc = detail.npc
	text = text .. ":32|t" .. sp1 .. npc .. "\r\r" .. white
	
	if locale == "ruRU" then
	
		text = text .. L["This level"] .. " " .. L["quest"] .. " " .. detail.l .. ", "
		
	else
	
		text = text .. L["This level"] .. " " .. detail.l .. " " .. L["quest"] .. " "
		
	end
	
	if detail.f == 1 then
	
		text = text .. L["starts a series with"]
		
	elseif detail.s == 1 then
	
		text = text .. L["is part of a series from"]
		
	else
	
		text = text .. L["starts with"]
		
	end
	
	text = text .. " " .. item .. " " .. L["in"] .. " "
	zone = Loremaster.Zones[tostring(detail.z)]
	text = text .. zone .. " " .. L["at"] .. " " .. blue .. detail.x .. white .. ", " .. blue .. detail.y .. white .. "."
	
	LoremasterInfoFrame.qid = detail.qid
	LoremasterInfoFrame.x = detail.x
	LoremasterInfoFrame.y = detail.y
	LoremasterInfoFrame.z = zone
	LoremasterInfoFrame.target = target
	LoremasterInfoBox:SetText(text)
	
end

function Loremaster:scanAchievements()

	local achlist = Loremaster[Loremaster.faction]
	local continent, ach, name, complete
	local achievements, c = {[1]={},[2]={},[3]={},[4]={},[5]={}}, ""
	local grey, white = "|cff778899", "|cffffffff"
	
	if not achlist then return end
	
	for continent = 1, 5 do
	
		for _, ach in pairs(achlist[continent]) do
		
			_, name, _, complete = GetAchievementInfo(ach)
			_, _, _, qty, req = GetAchievementCriteriaInfo(ach, 1)
			
			if complete then
			
				c = grey
				
			else
			
				c = white
				
			end
			
			table.insert(achievements[continent], {c .. name .. grey .. " (" .. (qty or 0) .. " / " .. (req or 0) .. ")", ach, complete})
			
		end
		
	end

	LoremasterDB.Achievements = achievements
	Loremaster.rescan = nil
	
end

function Loremaster:resetInterface()

	--reset the interface
	LoremasterQuestFrame.data = {}
	LoremasterQuestFrame.qdata = {}
	LoremasterZoneFrame.data = {}
	LoremasterContinentFrame.data = Loremaster:getContinents()
	Loremaster:updateQuestCount(-1)
	Loremaster:updateZoneCount(-1)
	LoremasterZoneFrame.selected = nil
	LoremasterZoneFrame.currentline = nil
	LoremasterQuestFrame.selected = nil
	LoremasterContinentFrame.selected = nil
	Loremaster:updateScroll(LoremasterContinentFramescroll)
	Loremaster:updateScroll(LoremasterQuestFramescroll)
	Loremaster:updateScroll(LoremasterZoneFramescroll)
	LoremasterInfoFrame.qid = nil
	LoremasterInfoFrame.x = nil
	LoremasterInfoFrame.y = nil
	LoremasterInfoFrame.z = nil
	LoremasterInfoFrame.target = nil
	LoremasterInfoBox:SetText("")
	Loremaster:resetLines(LoremasterContinentFrame)
	
end

function Loremaster:buildQuestList()

	local completed = GetQuestsCompleted()
	LoremasterDB.available = {["1"]={},["2"]={},["3"]={},["4"]={},["5"]={}}
	local qid, loaded, reason, c, zone, qs, faction, details, k, v, same
	local dups = {}

	
	--reset the interface
	Loremaster:resetInterface()
		
	--Try loading the data addon
	loaded, reason = LoadAddOn("Loremaster_Data")
	
	if not loaded then
        
		--load failed
		StaticPopup_Show("LOREMASTER_FAILED")
		return
		
	end
	
	if Loremaster.faction == "Alliance" then
	
		faction = 2
		
	else
	
		faction = 1
		
	end
	
	for c = 1, 5 do

		LoremasterDB.available[tostring(c)] = (Loremaster[continents[c]] or {})
	
		for zone, qs in pairs(LoremasterDB.available[tostring(c)]) do

			for qid, details in pairs(qs) do

				setmetatable(LoremasterDB.available[tostring(c)][zone], {__index = {0,0,3,0,0,"","",0,0,0,nil,nil}})
				--get rid of any quests for the opposite faction only
				if details[3] == faction then
				
					LoremasterDB.available[tostring(c)][zone][qid] = nil
					
				end
				
				--remove completed quests
				for k, v in pairs(completed) do
				
					if qid == k then

						--remove 'same as' quests
						if details[11] then
						
							for _, same in ipairs(details[11]) do
						
								LoremasterDB.available[tostring(c)][zone][same] = nil
								table.insert(dups, same)
								
							end
							
						end
						
						--remove completed quest
						LoremasterDB.available[tostring(c)][zone][k] = nil
						break
						
					end
					
				end
				
			end
		
		end

		Loremaster[continents[c]] = nil
		
	end
	
	--remove any duplicates from other zones
	for _, same in ipairs(dups) do
	
		Loremaster:removeQuest(same)
		
	end
	
	collectgarbage("collect")
	StaticPopup_Show("LOREMASTER_SUCCESS")
	
end